<?php declare(strict_types=1);

namespace Infomaker\Everyware\NewsML;

use Infomaker\Everyware\Support\NewRelicLog;
use Infomaker\Everyware\Support\Interfaces\PropertyObject;
use Infomaker\Everyware\NewsML\Parsers\ObjectParser;
use Exception;
use SimpleXMLIterator;

class NewsMLObjectTransformer implements Transformer
{

    public const VALID_TAG = 'object';

    /**
     * @var ObjectParser
     */
    private $parser;

    /**
     * @var string
     */
    private $newsmlPropertyKey;

    public function __construct(ObjectParser $parser, string $newsmlPropertyKey)
    {
        $this->parser = $parser;
        $this->newsmlPropertyKey = $newsmlPropertyKey;
    }

    private function parse(string $newsml): array
    {
        try {
            $iterator = new SimpleXMLIterator($newsml);

            if ($iterator->getName() === static::VALID_TAG) {
                return $this->parser->parse($iterator);
            }
        } catch (Exception $exception) {
            NewRelicLog::error(sprintf('Failed parsing NewsML from %s', $this->newsmlPropertyKey), $exception);
        }

        return [];
    }

    public function transform(PropertyObject $object): array
    {
        $newsml = $object->get($this->newsmlPropertyKey);
        if (empty($newsml)) {
            NewRelicLog::error(sprintf('Will not parse empty value: %s from Object: %s', $this->newsmlPropertyKey,
                $object->get('uuid')));
            return [];
        }

        return $this->parse($newsml);

    }
}
