<?php declare(strict_types=1);

namespace Infomaker\Everyware\NewsML\Parsers;

use Infomaker\Everyware\Support\Arr;
use SimpleXMLIterator;

/**
 * IdfParser
 *
 * @link    http://infomaker.se
 * @package Infomaker\Everyware\NewsML\Parsers
 * @since   Infomaker\Everyware\NewsML\Parsers\IdfParser 1.0.0
 */
class IdfParser implements ItemParser
{

    /**
     * @var array
     */
    protected $parsers;

    public function __construct(ElementParser $elementParser, ObjectParser $objectParser, GroupParser $groupParser)
    {
        $this->parsers = [
            ElementParser::TAG => $elementParser,
            ObjectParser::TAG => $objectParser,
            GroupParser::TAG => $groupParser
        ];
    }

    public function parse(SimpleXMLIterator $element): array
    {
        $items = [];

        foreach ($element as $tag => $item) {
            $parser = Arr::get($this->parsers, $tag, null);

            if ($parser instanceof ItemParser) {
                $items[] = $parser->parse($item);
            }
        }

        return Arr::flatten($items, 1);
    }
}
